@@ -21,9 +21,9 @@
 #include "llvm/CodeGen/SelectionDAGNodes.h"
 #include "llvm/CodeGen/TargetLowering.h"
 #include "llvm/CodeGen/ValueTypes.h"
+#include "llvm/CodeGenTypes/MachineValueType.h"
 #include "llvm/IR/CallingConv.h"
 #include "llvm/IR/InlineAsm.h"
-#include "llvm/Support/MachineValueType.h"
 #include <cstdint>
 #include <utility>
 
@@ -47,15 +47,25 @@ enum NodeType : unsigned {
   CALLnr,      // Function call that does not return.
   CALLR,
 
-  RET_FLAG,    // Return with a flag operand.
+  RET_GLUE,    // Return with a glue operand.
   BARRIER,     // Memory barrier.
   JT,          // Jump table.
   CP,          // Constant pool.
 
   COMBINE,
-  VASL,
+  VASL,        // Vector shifts by a scalar value
   VASR,
   VLSR,
+  MFSHL,       // Funnel shifts with the shift amount guaranteed to be
+  MFSHR,       // within the range of the bit width of the element.
+
+  SSAT,        // Signed saturate.
+  USAT,        // Unsigned saturate.
+  SMUL_LOHI,   // Same as ISD::SMUL_LOHI, but opaque to the combiner.
+  UMUL_LOHI,   // Same as ISD::UMUL_LOHI, but opaque to the combiner.
+               // We want to legalize MULH[SU] to [SU]MUL_LOHI, but the
+               // combiner will keep rewriting it back to MULH[SU].
+  USMUL_LOHI,  // Like SMUL_LOHI, but unsigned*signed.
 
   TSTBIT,
   INSERT,
@@ -67,6 +77,7 @@ enum NodeType : unsigned {
   EH_RETURN,
   DCFETCH,
   READCYCLE,
+  READTIMER,
   PTRUE,
   PFALSE,
   D2P,         // Convert 8-byte value to 8-bit predicate register. [*]
@@ -79,19 +90,30 @@ enum NodeType : unsigned {
   QCAT,
   QTRUE,
   QFALSE,
+
+  TL_EXTEND,   // Wrappers for ISD::*_EXTEND and ISD::TRUNCATE to prevent DAG
+  TL_TRUNCATE, // from auto-folding operations, e.g.
+               // (i32 ext (i16 ext i8)) would be folded to (i32 ext i8).
+               // To simplify the type legalization, we want to keep these
+               // single steps separate during type legalization.
+               // TL_[EXTEND|TRUNCATE] Inp, i128 _, i32 Opc
+               // * Inp is the original input to extend/truncate,
+               // * _ is a dummy operand with an illegal type (can be undef),
+               // * Opc is the original opcode.
+               // The legalization process (in Hexagon lowering code) will
+               // first deal with the "real" types (i.e. Inp and the result),
+               // and once all of them are processed, the wrapper node will
+               // be replaced with the original ISD node. The dummy illegal
+               // operand is there to make sure that the legalization hooks
+               // are called again after everything else is legal, giving
+               // us the opportunity to undo the wrapping.
+
   TYPECAST,    // No-op that's used to convert between different legal
                // types in a register.
   VALIGN,      // Align two vectors (in Op0, Op1) to one that would have
                // been loaded from address in Op2.
   VALIGNADDR,  // Align vector address: Op0 & -Op1, except when it is
                // an address in a vector load, then it's a no-op.
-  VPACKL,      // Pack low parts of the input vector to the front of the
-               // output. For example v64i16 VPACKL(v32i32) will pick
-               // the low halfwords and pack them into the first 32
-               // halfwords of the output. The rest of the output is
-               // unspecified.
-  VUNPACK,     // Unpacking into low elements with sign extension.
-  VUNPACKU,    // Unpacking into low elements with zero extension.
   ISEL,        // Marker for nodes that were created during ISel, and
                // which need explicit selection (would have been left
                // unselected otherwise).
@@ -111,8 +133,6 @@ public:
   explicit HexagonTargetLowering(const TargetMachine &TM,
                                  const HexagonSubtarget &ST);
 
-  bool isHVXVectorType(MVT Ty) const;
-
   /// IsEligibleForTailCallOptimization - Check whether the call is eligible
   /// for tail call optimization. Targets which want to do tail call
   /// optimization should implement this function.
@@ -129,8 +149,8 @@ public:
   bool isTruncateFree(Type *Ty1, Type *Ty2) const override;
   bool isTruncateFree(EVT VT1, EVT VT2) const override;
 
-  bool isCheapToSpeculateCttz() const override { return true; }
-  bool isCheapToSpeculateCtlz() const override { return true; }
+  bool isCheapToSpeculateCttz(Type *) const override { return true; }
+  bool isCheapToSpeculateCtlz(Type *) const override { return true; }
   bool isCtlzFast() const override { return true; }
 
   bool hasBitTest(SDValue X, SDValue Y) const override;
@@ -147,10 +167,14 @@ public:
   // Should we expand the build vector with shuffles?
   bool shouldExpandBuildVectorWithShuffles(EVT VT,
       unsigned DefinedValues) const override;
+  bool isExtractSubvectorCheap(EVT ResVT, EVT SrcVT,
+      unsigned Index) const override;
+
+  bool isTargetCanonicalConstantNode(SDValue Op) const override;
 
   bool isShuffleMaskLegal(ArrayRef<int> Mask, EVT VT) const override;
-  TargetLoweringBase::LegalizeTypeAction getPreferredVectorAction(MVT VT)
-      const override;
+  LegalizeTypeAction getPreferredVectorAction(MVT VT) const override;
+  LegalizeAction getCustomOperationAction(SDNode &Op) const override;
 
   SDValue LowerOperation(SDValue Op, SelectionDAG &DAG) const override;
   void LowerOperationWrapper(SDNode *N, SmallVectorImpl<SDValue> &Results,
@@ -177,12 +201,14 @@ public:
   SDValue LowerStore(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerUnalignedLoad(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerUAddSubO(SDValue Op, SelectionDAG &DAG) const;
-  SDValue LowerAddSubCarry(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerUAddSubOCarry(SDValue Op, SelectionDAG &DAG) const;
 
   SDValue LowerDYNAMIC_STACKALLOC(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerINLINEASM(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerFDIV(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerPREFETCH(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerREADCYCLECOUNTER(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerREADSTEADYCOUNTER(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerEH_LABEL(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerEH_RETURN(SDValue Op, SelectionDAG &DAG) const;
   SDValue
@@ -200,13 +226,13 @@ public:
   SDValue LowerToTLSLocalExecModel(GlobalAddressSDNode *GA,
       SelectionDAG &DAG) const;
   SDValue GetDynamicTLSAddr(SelectionDAG &DAG, SDValue Chain,
-      GlobalAddressSDNode *GA, SDValue InFlag, EVT PtrVT,
-      unsigned ReturnReg, unsigned char OperandFlags) const;
+      GlobalAddressSDNode *GA, SDValue InGlue, EVT PtrVT,
+      unsigned ReturnReg, unsigned char OperandGlues) const;
   SDValue LowerGLOBAL_OFFSET_TABLE(SDValue Op, SelectionDAG &DAG) const;
 
   SDValue LowerCall(TargetLowering::CallLoweringInfo &CLI,
       SmallVectorImpl<SDValue> &InVals) const override;
-  SDValue LowerCallResult(SDValue Chain, SDValue InFlag,
+  SDValue LowerCallResult(SDValue Chain, SDValue InGlue,
                           CallingConv::ID CallConv, bool isVarArg,
                           const SmallVectorImpl<ISD::InputArg> &Ins,
                           const SDLoc &dl, SelectionDAG &DAG,
@@ -223,7 +249,7 @@ public:
   bool CanLowerReturn(CallingConv::ID CallConv,
                       MachineFunction &MF, bool isVarArg,
                       const SmallVectorImpl<ISD::OutputArg> &Outs,
-                      LLVMContext &Context) const override;
+                      LLVMContext &Context, const Type *RetTy) const override;
 
   SDValue LowerReturn(SDValue Chain, CallingConv::ID CallConv, bool isVarArg,
                       const SmallVectorImpl<ISD::OutputArg> &Outs,
@@ -305,12 +331,12 @@ public:
   bool allowsMemoryAccess(LLVMContext &Context, const DataLayout &DL, EVT VT,
                           unsigned AddrSpace, Align Alignment,
                           MachineMemOperand::Flags Flags,
-                          bool *Fast) const override;
+                          unsigned *Fast) const override;
 
   bool allowsMisalignedMemoryAccesses(EVT VT, unsigned AddrSpace,
                                       Align Alignment,
                                       MachineMemOperand::Flags Flags,
-                                      bool *Fast) const override;
+                                      unsigned *Fast) const override;
 
   /// Returns relocation base for the given PIC jumptable.
   SDValue getPICJumpTableRelocBase(SDValue Table, SelectionDAG &DAG)
@@ -319,6 +345,9 @@ public:
   bool shouldReduceLoadWidth(SDNode *Load, ISD::LoadExtType ExtTy,
                              EVT NewVT) const override;
 
+  void AdjustInstrPostInstrSelection(MachineInstr &MI,
+                                     SDNode *Node) const override;
+
   // Handling of atomic RMW instructions.
   Value *emitLoadLinked(IRBuilderBase &Builder, Type *ValueTy, Value *Addr,
                         AtomicOrdering Ord) const override;
@@ -337,6 +366,7 @@ public:
 private:
   void initializeHVXLowering();
   unsigned getPreferredHvxVectorAction(MVT VecTy) const;
+  unsigned getCustomHvxOperationAction(SDNode &Op) const;
 
   bool validateConstPtrAlignment(SDValue Ptr, Align NeedAlign, const SDLoc &dl,
                                  SelectionDAG &DAG) const;
@@ -353,14 +383,21 @@ private:
                         SelectionDAG &DAG) const;
   SDValue extractVector(SDValue VecV, SDValue IdxV, const SDLoc &dl,
                         MVT ValTy, MVT ResTy, SelectionDAG &DAG) const;
+  SDValue extractVectorPred(SDValue VecV, SDValue IdxV, const SDLoc &dl,
+                            MVT ValTy, MVT ResTy, SelectionDAG &DAG) const;
   SDValue insertVector(SDValue VecV, SDValue ValV, SDValue IdxV,
                        const SDLoc &dl, MVT ValTy, SelectionDAG &DAG) const;
+  SDValue insertVectorPred(SDValue VecV, SDValue ValV, SDValue IdxV,
+                           const SDLoc &dl, MVT ValTy, SelectionDAG &DAG) const;
   SDValue expandPredicate(SDValue Vec32, const SDLoc &dl,
                           SelectionDAG &DAG) const;
   SDValue contractPredicate(SDValue Vec64, const SDLoc &dl,
                             SelectionDAG &DAG) const;
+  SDValue getSplatValue(SDValue Op, SelectionDAG &DAG) const;
   SDValue getVectorShiftByInt(SDValue Op, SelectionDAG &DAG) const;
   SDValue appendUndef(SDValue Val, MVT ResTy, SelectionDAG &DAG) const;
+  SDValue getCombine(SDValue Hi, SDValue Lo, const SDLoc &dl, MVT ResTy,
+                     SelectionDAG &DAG) const;
 
   bool isUndef(SDValue Op) const {
     if (Op.isMachineOpcode())
@@ -405,17 +442,45 @@ private:
   TypePair typeSplit(MVT Ty) const;
   MVT typeExtElem(MVT VecTy, unsigned Factor) const;
   MVT typeTruncElem(MVT VecTy, unsigned Factor) const;
+  TypePair typeExtendToWider(MVT Ty0, MVT Ty1) const;
+  TypePair typeWidenToWider(MVT Ty0, MVT Ty1) const;
+  MVT typeLegalize(MVT Ty, SelectionDAG &DAG) const;
+  MVT typeWidenToHvx(MVT Ty) const;
 
   SDValue opJoin(const VectorPair &Ops, const SDLoc &dl,
                  SelectionDAG &DAG) const;
   VectorPair opSplit(SDValue Vec, const SDLoc &dl, SelectionDAG &DAG) const;
   SDValue opCastElem(SDValue Vec, MVT ElemTy, SelectionDAG &DAG) const;
 
+  SDValue LoHalf(SDValue V, SelectionDAG &DAG) const {
+    MVT Ty = ty(V);
+    const SDLoc &dl(V);
+    if (!Ty.isVector()) {
+      assert(Ty.getSizeInBits() == 64);
+      return DAG.getTargetExtractSubreg(Hexagon::isub_lo, dl, MVT::i32, V);
+    }
+    MVT HalfTy = typeSplit(Ty).first;
+    SDValue Idx = getZero(dl, MVT::i32, DAG);
+    return DAG.getNode(ISD::EXTRACT_SUBVECTOR, dl, HalfTy, V, Idx);
+  }
+  SDValue HiHalf(SDValue V, SelectionDAG &DAG) const {
+    MVT Ty = ty(V);
+    const SDLoc &dl(V);
+    if (!Ty.isVector()) {
+      assert(Ty.getSizeInBits() == 64);
+      return DAG.getTargetExtractSubreg(Hexagon::isub_hi, dl, MVT::i32, V);
+    }
+    MVT HalfTy = typeSplit(Ty).first;
+    SDValue Idx = DAG.getConstant(HalfTy.getVectorNumElements(), dl, MVT::i32);
+    return DAG.getNode(ISD::EXTRACT_SUBVECTOR, dl, HalfTy, V, Idx);
+  }
+
   bool allowsHvxMemoryAccess(MVT VecTy, MachineMemOperand::Flags Flags,
-                             bool *Fast) const;
+                             unsigned *Fast) const;
   bool allowsHvxMisalignedMemoryAccesses(MVT VecTy,
                                          MachineMemOperand::Flags Flags,
-                                         bool *Fast) const;
+                                         unsigned *Fast) const;
+  void AdjustHvxInstrPostInstrSelection(MachineInstr &MI, SDNode *Node) const;
 
   bool isHvxSingleTy(MVT Ty) const;
   bool isHvxPairTy(MVT Ty) const;
@@ -441,8 +506,9 @@ private:
                               const SDLoc &dl, SelectionDAG &DAG) const;
   SDValue insertHvxElementPred(SDValue VecV, SDValue IdxV, SDValue ValV,
                                const SDLoc &dl, SelectionDAG &DAG) const;
-  SDValue extractHvxSubvectorReg(SDValue VecV, SDValue IdxV, const SDLoc &dl,
-                                 MVT ResTy, SelectionDAG &DAG) const;
+  SDValue extractHvxSubvectorReg(SDValue OrigOp, SDValue VecV, SDValue IdxV,
+                                 const SDLoc &dl, MVT ResTy, SelectionDAG &DAG)
+                                 const;
   SDValue extractHvxSubvectorPred(SDValue VecV, SDValue IdxV, const SDLoc &dl,
                                   MVT ResTy, SelectionDAG &DAG) const;
   SDValue insertHvxSubvectorReg(SDValue VecV, SDValue SubV, SDValue IdxV,
@@ -453,6 +519,20 @@ private:
                               bool ZeroExt, SelectionDAG &DAG) const;
   SDValue compressHvxPred(SDValue VecQ, const SDLoc &dl, MVT ResTy,
                           SelectionDAG &DAG) const;
+  SDValue resizeToWidth(SDValue VecV, MVT ResTy, bool Signed, const SDLoc &dl,
+                        SelectionDAG &DAG) const;
+  SDValue extractSubvector(SDValue Vec, MVT SubTy, unsigned SubIdx,
+                           SelectionDAG &DAG) const;
+  VectorPair emitHvxAddWithOverflow(SDValue A, SDValue B, const SDLoc &dl,
+                                    bool Signed, SelectionDAG &DAG) const;
+  VectorPair emitHvxShiftRightRnd(SDValue Val, unsigned Amt, bool Signed,
+                                  SelectionDAG &DAG) const;
+  SDValue emitHvxMulHsV60(SDValue A, SDValue B, const SDLoc &dl,
+                          SelectionDAG &DAG) const;
+  SDValue emitHvxMulLoHiV60(SDValue A, bool SignedA, SDValue B, bool SignedB,
+                            const SDLoc &dl, SelectionDAG &DAG) const;
+  SDValue emitHvxMulLoHiV62(SDValue A, bool SignedA, SDValue B, bool SignedB,
+                            const SDLoc &dl, SelectionDAG &DAG) const;
 
   SDValue LowerHvxBuildVector(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxSplatVector(SDValue Op, SelectionDAG &DAG) const;
@@ -467,27 +547,37 @@ private:
   SDValue LowerHvxZeroExt(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxCttz(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxMulh(SDValue Op, SelectionDAG &DAG) const;
-  SDValue LowerHvxSetCC(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerHvxMulLoHi(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxExtend(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxSelect(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxShift(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerHvxFunnelShift(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxIntrinsic(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxMaskedOp(SDValue Op, SelectionDAG &DAG) const;
   SDValue LowerHvxFpExtend(SDValue Op, SelectionDAG &DAG) const;
-  SDValue LowerHvxConvertFpInt(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerHvxFpToInt(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LowerHvxIntToFp(SDValue Op, SelectionDAG &DAG) const;
+  SDValue ExpandHvxFpToInt(SDValue Op, SelectionDAG &DAG) const;
+  SDValue ExpandHvxIntToFp(SDValue Op, SelectionDAG &DAG) const;
+
+  VectorPair SplitVectorOp(SDValue Op, SelectionDAG &DAG) const;
 
-  SDValue SplitHvxPairOp(SDValue Op, SelectionDAG &DAG) const;
   SDValue SplitHvxMemOp(SDValue Op, SelectionDAG &DAG) const;
   SDValue WidenHvxLoad(SDValue Op, SelectionDAG &DAG) const;
   SDValue WidenHvxStore(SDValue Op, SelectionDAG &DAG) const;
   SDValue WidenHvxSetCC(SDValue Op, SelectionDAG &DAG) const;
-  SDValue WidenHvxExtend(SDValue Op, SelectionDAG &DAG) const;
-  SDValue WidenHvxTruncate(SDValue Op, SelectionDAG &DAG) const;
+  SDValue LegalizeHvxResize(SDValue Op, SelectionDAG &DAG) const;
+  SDValue ExpandHvxResizeIntoSteps(SDValue Op, SelectionDAG &DAG) const;
+  SDValue EqualizeFpIntConversion(SDValue Op, SelectionDAG &DAG) const;
+
+  SDValue CreateTLWrapper(SDValue Op, SelectionDAG &DAG) const;
+  SDValue RemoveTLWrapper(SDValue Op, SelectionDAG &DAG) const;
 
   std::pair<const TargetRegisterClass*, uint8_t>
   findRepresentativeClass(const TargetRegisterInfo *TRI, MVT VT)
       const override;
 
+  bool shouldSplitToHvx(MVT Ty, SelectionDAG &DAG) const;
   bool shouldWidenToHvx(MVT Ty, SelectionDAG &DAG) const;
   bool isHvxOperation(SDNode *N, SelectionDAG &DAG) const;
   SDValue LowerHvxOperation(SDValue Op, SelectionDAG &DAG) const;
@@ -495,7 +585,14 @@ private:
                                 SelectionDAG &DAG) const;
   void ReplaceHvxNodeResults(SDNode *N, SmallVectorImpl<SDValue> &Results,
                              SelectionDAG &DAG) const;
-  SDValue PerformHvxDAGCombine(SDNode *N, DAGCombinerInfo &DCI) const;
+
+  SDValue combineTruncateBeforeLegal(SDValue Op, DAGCombinerInfo &DCI) const;
+  SDValue combineConcatVectorsBeforeLegal(SDValue Op, DAGCombinerInfo & DCI)
+      const;
+  SDValue combineVectorShuffleBeforeLegal(SDValue Op, DAGCombinerInfo & DCI)
+      const;
+
+  SDValue PerformHvxDAGCombine(SDNode * N, DAGCombinerInfo & DCI) const;
 };
 
 } // end namespace llvm
